% Function to create a hankel matrix for a vector time series data
% Inputs
% M: d x N matrix where d = dimension of the vector, N being the
% length of the time series. 
% r: degree of overlap 
%function [H, desc, desc2, desc3] = getHankelMat(M, r, ne)
function [H, desc, desc2, desc3] = getHankelMat(M, r, ne)
  [d , N] = size(M);
  % number of columns
  p = N - r+1;
  H = [];
  for j=1:p
    h = M(:,j:j-1+r);
    H = [H, h(:)];  
  end
  
  % Getting the descriptor based on the original hankel matrix
  L = H*H';
  q = trace(L*L');
  H  = H/sqrt(q);
  D = H*H';
  [U V ~] = svd(D);
  % reproject D into ne eigen vectors - rearrange these to compute the
  % descriptor
  desc = [];
  for j=1:ne
    desc = [desc; (U(:, j)'*D)'];
  end
    
  % Getting the descriptor based on the low rank decomposition
  % The hankel matrices could be corrupt with noise, so lets find the low
  % rank component
%   if rank(H) > 5
%     lambda = .5/sqrt(size(H,1)) ; % change .5 in order control the rank
%     tol = 1.0000e-006;
%     maxIter = 1000;
%     path('./RASL_toolbox/', path);
%     [A, E, numIter] = rasl_inner_ialm_noT(H, lambda, tol, maxIter);
%     
%     % Based on the low rank component
%     L = A*A';
%     q = trace(L*L');
%     A  = A/sqrt(q);
%     D = A*A';
%     [U V ~] = svd(D);
%         
%     desc2 = [];
%     for j=1:ne
%       desc2 = [desc2; (U(:, j)'*D)'];
%     end
%     % Based on the error component
%     L = E*E';
%     q = trace(L*L');
%     E  = E/sqrt(q);
%     D = E*E';
%     [U V ~] = svd(D);
%         
%     desc3 = [];
%     for j=1:ne
%       desc3 = [desc3; (U(:, j)'*D)'];
%     end
%     
%   else
%     desc2 = desc;  
%     desc3 = desc;
%   end
return