clear all;
clc;
close all;
IN_DIR = './data';
TOOLBOX = './tools/';
file = 'EC.mat';
load([IN_DIR '/' file]);
labels = dlmread([IN_DIR '/EC-labels.txt'], '/');
nsplits = 5;

% Preparing data for Max Pooling SVM 

DATA =[];
IDS = [];
for i =1:size(TSdata,2)
  fprintf(1, 'Maxpooling video id: %d\n', TSdata(i).id);
  if ~isempty(TSdata(i).seq)
    s = labels(labels(:,2)==TSdata(i).id, :);
    if ~isempty(s)
      [~, idx] = max(TSdata(i).seq);
      IDS = [IDS; s(1) s(2)];
      % if sequence Length is 1
      if size(TSdata(i).seq, 2) ==1
        DATA = [DATA; TSdata(i).seq'];  
      else
        DATA = [DATA; max(TSdata(i).seq, [], 2)'];
      end
    end
  end  
end
% Loading EC2 data
file = 'EC2.mat';
load([IN_DIR '/' file]);
labels = dlmread([IN_DIR '/EC2-labels.txt'], '/');

for i =1:size(TSdata,2)
  fprintf(1, 'Maxpooling video id: %d\n', TSdata(i).id);
  if ~isempty(TSdata(i).seq)
    s = labels(labels(:,2)==TSdata(i).id, :);
    if ~isempty(s)
      [~, idx] = max(TSdata(i).seq);
      IDS = [IDS; s(1) s(2)];
      % if sequence Length is 1
      if size(TSdata(i).seq, 2) ==1
        DATA = [DATA; TSdata(i).seq'];  
      else
        DATA = [DATA; max(TSdata(i).seq, [], 2)'];
      end
    end
  end  
end

% Generating the splits
IDS = [IDS, mod(IDS(:,2), nsplits) + 1];

% Perform Classification for each split using SVM
classes = unique(sort(IDS(:,1)));
nclasses = size(classes,1);
model = struct();
path([TOOLBOX '/libsvm-3.1-hi-chi/matlab/'], path);
svmParams = '-q -t 0 -d 2 -b 1';
        
RES = [];
for i=1:nsplits
  % Testing data for ith split
  tsDATA = DATA(IDS(:,3)==i,:);
  tsIDS  = IDS(IDS(:,3)==i,:);
  % Training data for ith split
  trDATA = DATA(IDS(:,3)~=i,:);
  trIDS  = IDS(IDS(:,3)~=i,:);
  
  % Perform training for each event model
  for j=1:nclasses
    cl_id = classes(j);
    fprintf(1, 'Split: %d __ Training SVM for model %d\n', i, cl_id);
    trPOS = trDATA(trIDS(:,1)==cl_id, :);
    trNEG = trDATA(trIDS(:,1)~=cl_id, :);
    nsamples = size(trPOS,1);
    negIdx = randperm(size(trNEG, 1));
    trNEG = trNEG(negIdx(1:nsamples),:);
    model(cl_id).svm = svmtrain([ones(nsamples, 1); zeros(nsamples, 1)], ...
        [trPOS; trNEG] , svmParams);
  end
  
  % Perform testing
  for j=1:nclasses
    cl_id = classes(j);
    fprintf(1, 'Split: %d __ Testing SVM for class %d\n', i, cl_id);
    tsPOS = tsDATA(tsIDS(:,1)==cl_id,:);
    r =[];
    for k=1:nclasses
      [~, ~, c] = svmpredict(ones(size(tsPOS,1),1), tsPOS, model(classes(k)).svm,'-b 1');
      r = [r, c(:,1)];
    end
    [~, idxs] = max(r');
    % Predicted Label, True Label, Video ID, SPLIT ID, Event Probabilities
    RES = [RES; idxs', tsIDS(tsIDS(:,1)==cl_id,:), r];
  end
end

% different flags for different dataset combinations
% EC1 and EC2 combined
RES(RES(:,1)>15) = RES(RES(:,1)>15)+5;

X = confusionmat(RES(:,2), RES(:,1));
ACC = sum(diag(X))/sum(sum(X));
save([IN_DIR '/SVM_MAXP_EC-' file], 'RES');